#
#   These functions govern the insertion and discovery of data in the
#  BigSur Grid section of the schema. These functions all take a
#  set of information and return an oid, which is either the
#  value of the row just inserted, or the value of a row which
#  matches (or is near enough to) the values passed in.
#
#  Get_Axis_Name - Takes the GCM canonical Axis attributes and returns a Name.
#                  If the Axis asked for does not exist, then the return 
#                  value is an error.
#
proc Get_Axis_Name { axisType axisinfoUnits axisInfoSpacing axisOrigin
axisAccuracy axisCount } {

global MD
set Exists ""

set UMax [ VMax $axisInfoSpacing $axisAccuracy ]
set UMin [ VMin $axisInfoSpacing $axisAccuracy ]
set OMax [ VMax $axisOrigin $axisAccuracy ]
set OMin [ VMin $axisOrigin $axisAccuracy ]
set AMax [ VMax $axisAccuracy 0.01 ]
set AMin [ VMin $axisAccuracy 0.01 ]
set Qry "
select Abstract
  from Axis
 where axisType = '$axisType'
   and axisCount = $axisCount
   and axisinfoUnits = '$axisinfoUnits'
   and axisInfoSpacing between '$UMin'::double precision
                           and '$UMax'::double precision
   and axisOrigin between '$OMin'::double precision
                           and '$OMax'::double precision
   and axisAccuracy between '$AMin'::double precision
                           and '$AMax'::double precision;"

 set Exists [ mi_select_rows $MD(connection) $Qry ]

 if { $Exists != "" } { return $Exists}

 return "ERROR: Not found"
}
#
#  Add_Axis - Takes the Axis attributes and returns an oid which 
#             corresponds to a row in the relation with these values.
#           
#     Add_Axis LONGITUDE Spatial Degrees 5.0 -180.0 0.005 72
#
proc Add_Axis { Abstract axisType axisinfoUnits axisInfoSpacing axisOrigin
axisAccuracy axisCount } {

global MD
set Exists ""

set UMax [ VMax $axisInfoSpacing $axisAccuracy ]
set UMin [ VMin $axisInfoSpacing $axisAccuracy ]
set OMax [ VMax $axisOrigin $axisAccuracy ]
set OMin [ VMin $axisOrigin $axisAccuracy ]
set AMax [ VMax $axisAccuracy 0.01 ]
set AMin [ VMin $axisAccuracy 0.01 ]

set Qry "
select oid
  from Axis
 where axisName = '$Abstract'
   and axisType = '$axisType'
   and axisCount = $axisCount
   and axisinfoUnits = '$axisinfoUnits'
   and axisInfoSpacing between '$UMin'::double precision 
                           and '$UMax'::double precision
   and axisOrigin between '$OMin'::double precision 
                           and '$OMax'::double precision
   and axisAccuracy between '$AMin'::double precision 
                           and '$AMax'::double precision;" 

 set Exists [ mi_select_rows $MD(connection) $Qry ]
 if { $Exists != "" } { return $Exists }

 set AddQuery "
insert into Axis
		(axisName, axisType, axisCount, axisinfoUnits,
		axisInfoSpacing, axisOrigin,
		axisAccuracy)
values
		('$Abstract', '$axisType', $axisCount, '$axisinfoUnits',
		  '$axisInfoSpacing'::double precision, '$axisOrigin'::double precision,
		  '$axisAccuracy'::double precision);"

mi_exec $MD(connection) $AddQuery

set Exists [ mi_select_rows $MD(connection) $Qry ]

 if { $Exists != "" } { return $Exists }
 return "ERROR: failed 
$AddQuery
or
$Qry"


}
#
#  Add_Grid - Takes the attribute information identifying a Grid and an
#             ordered set of Axis and returns an oid corresponding to
#             a tuple in this relation with these values.
#
#             If no matching tuple found, one is added and the oid of this
#             new tuple is returned.
#
proc Add_Grid { productGridName productGridAxisCount originCorner \
cellPositionAtCorner  productGridAxes } {


 set Exists ""
 global MD

 set prodGridCnt [ llength $productGridAxes ]

 if { $prodGridCnt != $productGridAxisCount } { return "ERROR: number of Axes specified siffers from number passed" }

 set Qry "
select distinct G.oid
  from GCMGrid G
 where G.gridAxisCount = $productGridAxisCount"

 set Seq 1 
 foreach i $productGridAxes {
  set Qry "$Qry
and EXISTS ( select 1
               from GCMAxis A$Seq, GGridAxisRes R$Seq
              where deref(R$Seq.Axis).oid        = A$Seq.oid
                and deref(R$Seq.ProductGrid).oid = G.oid
                and A$Seq.oid = '$i'::oid
                and R$Seq.seqNum = $Seq)"
  incr Seq
 }
 set Qry "$Qry;"
 set Exists [ mi_select_rows $MD(connection) $Qry ]
 if { $Exists != "" } { return $Exists }
#
#  OK. No matching Grid found. See if one can be added.
#
 set GetOIDQry "
select G.oid
  from GCMGrid G
 where G.gridName = '$productGridName'
   and G.gridAxisCount = $productGridAxisCount;"

 set Exists ""
 set Exists [ mi_select_rows $MD(connection) $GetOIDQry ]
 if { $Exists != "" } { return "ERROR: same Grid Name and ordinality applies to another Grid" }

 set InsertQry "
insert into GCMGrid 
( gridName, gridAxisCount, originCorner, cellPositionAtCorner )
values
( '$productGridName','$productGridAxisCount'::integer, '$originCorner', 
'$cellPositionAtCorner'::boolean);"

  mi_exec $MD(connection) $InsertQry

#
# Now get the new productGrid's oid
#
 set GridOID [ mi_select_rows $MD(connection) $GetOIDQry ]
 set Seq 1
 set InsertQry ""

 foreach i $productGridAxes {
#
#  Check that the Axis exists before you fuck with it.
#
  set ChkOID "
select 1
  from Axis
 where oid = '$i'::oid;"

  set Exists ""
  set Exists [ mi_select_rows $MD(connection) $ChkOID ]
  if { $Exists == "" } { return "ERROR: Specified Axis does not exist" }
#
#  Now do the Axis' insert
#
   set InsertQry "$InsertQry
insert into GGridAxisRes
( ProductGrid, Axis, seqNum )
select ref(G),
       ref(A),
       $Seq
  from GCMGrid G, GCMAxis A
 where G.oid = '$GridOID'::oid
   and A.oid = '$i'::oid;"

 incr Seq
 }

  mi_exec $MD(connection) $InsertQry

  set Exists ""
  set Exists [ mi_select_rows $MD(connection) $Qry ]

  if { $Exists == "" } { return "ERROR: Unable to find or add new Grid" }

  return $Exists
}

