#
# init.tcl - initialization of the MiTcl library interface
#
# (vi users: ":set tabstop=4" for better readability)
#

#
# mi_valid_connection - test to see if a connection id is valid
#
# NB:  connection ids returned from mi_open or mi_server_connect
# are never reused
#
proc mi_valid_connection {conn} {
	if [catch {mi_get_connection_info $conn}] {
		return 0
	} else {
		return 1
	}
}


proc mi_connect {host port db user password} {
	set conn [mi_server_connect $host $port]
	mi_login $conn $db $user $password
	return $conn
}

#
# mi_row_values - return as a list the values of a row
#
# flat: = 1 "flatten" out composite types to on level
#		= 0 return composite types as lists of lists [of lists ...]
#
proc mi_row_values {cid {flat 1}} {
	set rows ""
	while {[set row [mi_next_row $cid]] != ""} {
		lappend rows [mi_get_row_values $row $flat]
	}
	return $rows
}


#
# mi_column_values - return the values as a list of columns
#
# flat:	past to mi_row_values (see above)
#
proc mi_column_values {cid colnames {flat 1}} {
	if {$colnames == ""} {
		set colnames [mi_column_names $cid $flat]
	}
	set rows [mi_row_values $cid $flat]
	set values ""
	foreach name $colnames {
		set cnum [lsearch [mi_column_names $cid $flat] $name]
		if {$cnum == -1} {
			error "no such column:  $name"
		}
		set column ""
		foreach row $rows {
			lappend column [lindex $row $cnum]
		}
		lappend values $column
	}
	return $values
}


#
# mi_column_names - return the names of the first row of data
#
# flat:	past to mi_row_values (see above)
#
proc mi_column_names {cid {flat 1}} {
	mi_get_column_names [mi_get_row_desc_without_row $cid] $flat
}


#
# mi_column_names
#
proc mi_get_column_names {rdesc {flat 1}} {
	set names ""
	loop n 0 [mi_column_count $rdesc] {
		if {$flat && [mi_column_is_composite $rdesc $n]} {
			set cdesc [mi_composite_column_desc $rdesc $n]
			foreach name [mi_get_column_names $cdesc $flat] {
				lappend names $name
			}
		} else {
			lappend names [mi_column_name $rdesc $n]
		}
	}
	return $names
}

proc mi_select_rows {cid query {flat 1}} {
	mi_exec $cid $query
	mi_row_values $cid $flat
}

proc mi_select_first_row {cid query {flat 1}} {
	mi_exec $cid $query
	lindex [mi_row_values $cid $flat] 0
}

proc mi_select_columns {cid query {colnames ""} {flat 1}} {
	mi_exec $cid $query
	mi_column_values $cid $colnames $flat
}

proc mi_select_column {cid query colname {flat 1}} {
	mi_exec $cid $query
	lindex [mi_column_values $cid $colname $flat] 0
}

proc mi_select_colnames_and_rows {cid query {flat 1}} {
	mi_exec $cid $query
	set values [mi_column_names $cid $flat]
	lappend values [mi_row_values $cid $flat]
}

proc mi_exec_check {cid query type} {
	if {[mi_exec $cid $query] != $type} {
		mi_query_finish $cid
		error "Query did not return $type"
	}
}
	
proc mi_select {cid query} {mi_exec_check $cid $query MI_ROWS}
proc mi_dml {cid query} {mi_exec_check $cid $query MI_DML}
proc mi_ddl {cid query} {mi_exec_check $cid $query MI_DDL}

proc mi_0or1row {cid query {flat 1}} {
	mi_select $cid $query
	set row [mi_next_row $cid]
	if {$row == ""} {return ""}
	set values [mi_get_row_values $row $flat]
	if {[mi_next_row $cid] != ""} {
		mi_query_finish $cid
		error "query returned more than one row"
	}
	return $values
}

proc mi_1row {cid query {flat 1}} {
	set values [mi_0or1row $cid $query $flat]
	if {$values == ""} {
		error "query did not return a row"
	}
	return $values
}
