#
# mitcl_client.tcl - initialization script for mitcl RPC clients
#
#
# NB:  server, database, and possibly tmpdir are set in the miclient
# global variable before this file is sourced


#
# tkerror - dump background errors to stderr
#
proc tkerror err {
	global errorInfo
	puts stderr "$err\n\n$errorInfo"
}



#
# miclient_connected - check to see if the connection is open
#
proc miclient_connected {} {
	global miclient

	if [catch {mi_get_connection_info $miclient(connection)}] {
		return 0
	} else {
		return 1
	}
}


#
# miclient_open - connect to the server and login to the database
#
proc miclient_open {} {
	global miclient

	if ![miclient_connected] {
		set miclient(connection) [mi_server_connect $miclient(server)]
		mi_login $miclient(connection) $miclient(database)
	}
	return $miclient(connection)
}


#
# miclient_close - disconnect from the database
#
proc miclient_close {} {
	global miclient

	catch {mi_close $miclient(connection)}
}


#
# miclient_mkserver - create the RPC server
#
proc miclient_mkserver {} {
	global miclient

	# make the rpc server 
	set rpc [dp_MakeRPCServer 0 dp_CheckHost none 1]
	set miclient(rpcFile) [lindex $rpc 0]
	set miclient(rpcPort) [lindex $rpc 1]
}


#
# miclient_register - register a package
#
proc miclient_register {pkg} {
	global miclient

	# make sure we're an RPC server
	if ![info exists miclient(rpcPort)] {
		miclient_mkserver
	}

	# create the client composite type
	set miclient(client) "('$miclient(hostname)', $miclient(rpcPort))"

	# connect to the server and setup access control list
	miclient_open

	# register this package
	set query "insert into Clients.Hosts (
		client,
		machine,
		pkg
	) values (
		$miclient(client),
		$miclient(machine),
		'$pkg'
	);"
	mi_exec $miclient(connection) $query
}


#
# miclient_unregister - remove all package registrations
#
proc miclient_unregister {} {
	global miclient

	if ![info exists miclient(client)] {
		return
	}
	miclient_open
	mi_exec $miclient(connection) "delete from Clients.Hosts
		where client = $miclient(client);"
}


#
# miclient_atclose - procedure to make sure we unregister before we die
#
proc miclient_atexit {} {
	global miclient
	catch {close $miclient(rpcFile)}
	catch {miclient_unregister}
	catch {miclient_close}
}





#
#
# Eos Altarch specific stuff:
#
# NB:  These procedures expect to run from within
# an mitcl_client interpreter and expect to manipulate
# the database on the open miclient(connection)
# connection.
#

proc eos_provide_lazy {function} {
	global miclient
	miclient_open
	eos_get_proc $miclient(connection) $function
	miclient_register $function
	dp_atexit prepend "eos_progress_stop $miclient(connection) $function"
	eos_progress_update $miclient(connection) $function Waiting
}

#
# procedures for running an "eager evaluation" mitcl client
#
#
# eos_eager_poll - poll for alerts and reschedule for future polling
#
proc eos_eager_poll {{when 0}} {
	global miclient
	if $miclient(stop) {
		set miclient(stop) 0
		return
	}
	if !$miclient(busy) {
		mi_exec $miclient(connection) "poll;"
	}
	if {$when != 0} {
		dp_after $when eos_eager_poll $when
	}
}

#
# eos_eager_set - mark the client to execute the "do" procedure
#	after the "on" procedure's alerter fires, extracting
#	"do" from the database if needed
#
proc eos_eager_set {on do} {
	global miclient

	# make sure we're connected
	miclient_open

	# get "do" from the database if needed
	eos_get_proc $miclient(connection) $do

	# mark to do "do" after "on"
	lappend miclient(do,$on) $do

	# listen for "on"
	mi_exec $miclient(connection) "listen ${on}_alert;"

	# notify progress we're waiting
	eos_progress_update $miclient(connection) $do "Waiting"
	dp_atexit delete "eos_progress_stop $miclient(connection) $do"
	dp_atexit prepend "eos_progress_stop $miclient(connection) $do"
}

#
# eos_eager_start - register the eager_alert procedure and begin
#	polling for alerts
#
proc eos_eager_start {{when 30000}} {
	global miclient
	mi_add_callback Alerter_Fire_Msg eos_eager_alert
	set miclient(busy) 0
	set miclient(stop) 0
	eos_eager_poll $when
}

#
# eos_eager_stop - stop polling for alerts
#
proc eos_eager_stop {} {
	global miclient
	set miclient(stop) 1
}

#
# eos_eager_alert - find out the "on" procedure from the name
#	of the alert which fired and execute the cooresponding
#	"do" procedure on any objects which are waiting between
#	"on" and "do"
#
proc eos_eager_alert {conn alert_name alert_data} {
	global miclient

	regsub _alert [keylget alert_data name] "" on

	foreach do $miclient(do,$on) {
		foreach dlobh [mi_select_rows $conn \
			"select * from EosMeta.WaitingBetween('$on.%', '$do.%');"] {
			$do $dlobh
			eos_progress_update $conn $do "Waiting"
		}
	}
}

#
# eos_eager_provice - use eos_eager_set and then eos_eager_start
#	for one-line startup
#
proc eos_eager_provide {args} {
	set n 0
	while 1 {
		set on [lindex $args $n]
		set do [lindex $args [incr n]]
		if {($on == "") || ($do == "")} break
		eos_eager_set $on $do
	}
	eos_eager_start
}

#
#
# Procedures for Terascan database procedures
#
# eos_terascan_start - check the lineage tables to see if the result
#	of a Terascan function has been run on a particular output.
#	If so, set the output in the "lid" variable and return 0.
#	Otherwise, set the input and output filenames and start
#	a lineage entry and set the lineage entry id in the "lid"
#	variable and return 1
#
proc eos_terascan_start {function params input lVar iVar oVar} {
	global miclient

	upvar $lVar lid
	upvar $iVar input_file
	upvar $oVar output_file

	# setup the terascan environment
	eos_terascan_setup

	# make sure we're connected
	miclient_open

	# see if the output has already been created
	eos_progress_update $miclient(connection) $function \
		"Checking: $input"
	set lid [eos_get_dlobh $miclient(connection) $function $params $input]
	if {$lid != ""} {
		# return 0 - the procedure already worked on this input
		eos_progress_update $miclient(connection) $function \
			Waiting
		return 0
	}

	# get the input filename
	set input_file [mi_dlobh_to_file $input /tmp/$function.tmp]

	# construct the output filename
	set output_file [file root $input_file].$function

	# notify the progress watcher
	eos_progress_update $miclient(connection) $function \
		"Processing:  $input_file"

	# get a lineage entry id
	set lid [eos_log_start $miclient(connection) $function $params $input]

	# save the function and output_file of this lid
	set miclient(function,$lid) $function
	set miclient(output_file,$lid) $output_file

	# return 1 - the procedure will have to do work
	return 1
}

#
# eos_terascan_finish - log successful completion
#
proc eos_terascan_finish {lid} {
	global miclient

	# tell the progress watcher we're loading
	eos_progress_update $miclient(connection) $miclient(function,$lid) \
	"Loading:  $miclient(output_file,$lid)"

	# connect to the data server and load the output file
	set dataConn [eos_daac_login]
	set output [eos_load_tdf $miclient(connection) $dataConn $miclient(output_file,$lid)]
	mi_close $dataConn

	# commit the lineage information
	eos_log_finish $miclient(connection) $lid $output

	# tell the progress watcher we're done
	eos_progress_update $miclient(connection) $miclient(function,$lid) \
		Waiting

	# return the new dlobh
	return $output
}

#
# eos_terascan_finish - log a fuck up
#
proc eos_terascan_error {lid {err ""}} {
	global miclient

	# tell the progress watcher there was an error	
	eos_progress_update $miclient(connection) $miclient(function,$lid) \
	"ERROR!!!  Fail to process:  $miclient(output_file,$lid)"

	# commit the error lineage information
	eos_log_error $miclient(connection) $lid

	# tell the progress watcher we're done
	eos_progress_update $miclient(connection) $miclient(function,$lid) \
		"Waiting (Note:  last process returned error)"

	# return a Tcl error
	error "$miclient(function,$lid) failed:  $err"
}


#
# Startup:
#
# Get machine info, handle signals, register any packages
# named on the command line, and go into daemon mode if needed
#

set miclient(hostname) [exec hostname]

set uname [exec uname -a]
set miclient(machine) "('[lindex $uname 4]','[lindex $uname 0]',\
			'[lindex $uname 2]')"
unset uname

if ![info exists miclient(tmpdir)] {
	set miclient(tmpdir) /tmp
}

signal trap [list INT HUP QUIT TERM] {
	puts stderr "Caught %S, exiting..."
	miclient_atexit
	exit
}

dp_atexit set miclient_atexit

#
# In daemon mode, close and reopen stdout/stderr on a log file
#
if $miclient(daemon) {
	miclient_open
	miclient_mkserver
	if ![info exists miclient(logfile)] {	
		set miclient(logfile) /tmp/miclient.pid:[pid].port:$miclient(rpcPort)
	}
	if {[file dirname $miclient(logfile)] == "."} {
		set miclient(logfile) [pwd]/$miclient(logfile)
	}
	close stdout
	close stderr
	if {[open $miclient(logfile) w] != "stdout"} {
		error "Expected to open stdout!"
	}
	dup stdout stderr
	fcntl stdout linebuf 1
	fcntl stderr linebuf 1

	# change dir to /
	cd /
}
